package jrummikub.control;

import java.util.ArrayList;
import java.util.List;

import jrummikub.model.GameSettings;
import jrummikub.model.GameState;
import jrummikub.model.IRoundState;
import jrummikub.model.RoundState;
import jrummikub.model.Score;
import jrummikub.util.Connection;
import jrummikub.util.IListener;
import jrummikub.util.IListener1;
import jrummikub.view.IView;

/**
 * Controls a Game, at some point including all Rounds, starts new Rounds
 */
public class GameControl {
	private GameSettings gameSettings;
	private IView view;
	private RoundControl roundControl;
	private GameState gameState;
	private List<Connection> connections = new ArrayList<Connection>();

	/**
	 * Constructor
	 * 
	 * @param gameSettings
	 *          the game settings
	 * @param view
	 *          the view
	 */
	public GameControl(GameSettings gameSettings, IView view) {
		this.gameSettings = gameSettings;
		this.view = view;
		
		gameState = new GameState();
		gameState.setFirstRoundFirstPlayer((int)(Math.random() * gameSettings.getPlayerList().size()));

		connections.add(view.getNewRoundEvent().add(new IListener() {
			@Override
			public void handle() {
				startRound();
			}
		}));
		connections.add(view.getFinalScoreEvent().add(new IListener() {
			@Override
			public void handle() {
				finalScore();
			}
		}));
	}

	/**
	 * Game gets started by initializing the first Round
	 */
	public void startGame() {
		startRound();
	}

	private void startRound() {
		if (roundControl != null) {
			return;
		}

		IRoundState roundState = new RoundState(gameSettings);

		// TODO: add number of already played rounds
		roundState.setActivePlayerNumber(gameState.getFirstRoundFirstPlayer());
		
		roundControl = new RoundControl(roundState, view);
		roundControl.getEndOfRoundEvent().add(new IListener1<Score>() {

			@Override
			public void handle(Score roundScore) {
				endOfRound();
			}
		});

		roundControl.startRound();
	}
	
	private void endOfRound() {
		roundControl = null;
		view.enableWinPanel(true);
	}

	private void finalScore() {
		System.exit(0);
	}

}
